@@ -5,7 +5,8 @@
  * Thanks to the lunar authors:
  * Fung F. Lee	 <lee@umunhum.stanford.edu>
  * Ricky Yeung	 <cryeung@hotmail.com>
- * because algorithm from lunar: http://packages.debian.org/unstable/utils/lunar
+ * because algorithm from lunar:
+ * http://packages.debian.org/unstable/utils/lunar
  *
  * Special thanks to
  * redwooods <redwooods@gmail.com>
@@ -13,7 +14,7 @@
  * */
 
 #if HAVE_CONFIG_H
-    #include <config.h>
+#include <config.h>
 #endif
 #include <glib/gi18n-lib.h>
 #include <string.h>
@@ -86,11 +87,28 @@ static gchar* lunar_date_get_cal_holiday (LunarDate *date, gint max_len);
 
 G_DEFINE_TYPE (LunarDate, lunar_date, G_TYPE_OBJECT);
 
-static void
-lunar_date_class_init (LunarDateClass *class)
+static void lunar_date_finalize (GObject *gobject)
+{
+    LunarDate *date;
+
+    date = LUNAR_DATE (gobject);
+    if (date->holiday_solar != NULL) {
+        g_hash_table_destroy (date->holiday_solar);
+    }
+    if (date->holiday_lunar != NULL) {
+        g_hash_table_destroy (date->holiday_lunar);
+    }
+    if (date->holiday_week != NULL) {
+        g_hash_table_destroy (date->holiday_week);
+    }
+    G_OBJECT_CLASS (lunar_date_parent_class)->finalize (gobject);
+}
+
+static void lunar_date_class_init (LunarDateClass *class)
 {
     GObjectClass *gobject_class = G_OBJECT_CLASS (class);
 
+    gobject_class->finalize = lunar_date_finalize;
     gobject_class->set_property = lunar_date_set_property;
     gobject_class->get_property = lunar_date_get_property;
 }
@@ -106,12 +124,16 @@ static void lunar_date_init_holiday (LunarDate *date)
     gsize         size, len;
     gchar         *p;
 
-    date->holiday_solar = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
-    date->holiday_lunar = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
-    date->holiday_week = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
+    date->holiday_solar = g_hash_table_new_full(g_str_hash, g_str_equal,
+                                                g_free, g_free);
+    date->holiday_lunar = g_hash_table_new_full(g_str_hash, g_str_equal,
+                                                g_free, g_free);
+    date->holiday_week = g_hash_table_new_full(g_str_hash, g_str_equal,
+                                               g_free, g_free);
 
     resource = lunar_date_get_resource();
-    bytes = g_resource_lookup_data (resource, "/org/moses/lunar/holiday.dat", G_RESOURCE_LOOKUP_FLAGS_NONE, NULL);
+    bytes = g_resource_lookup_data (resource, "/org/moses/lunar/holiday.dat",
+                                    G_RESOURCE_LOOKUP_FLAGS_NONE, NULL);
     data = g_bytes_get_data (bytes, &size);
 
     keyfile = g_key_file_new();
@@ -124,15 +146,19 @@ static void lunar_date_init_holiday (LunarDate *date)
         while(j < len) {
             g_autofree gchar *value;
 
-            value = g_key_file_get_locale_string (keyfile, groups[i], keys[j], NULL, NULL);
+            value = g_key_file_get_locale_string (keyfile, groups[i], keys[j],
+                                                  NULL, NULL);
             p = strchr(keys[j], '[');
             *p = '\0';
             if (g_ascii_strcasecmp(groups[i], "SOLAR") == 0 ) {
-                g_hash_table_insert(date->holiday_solar, g_strdup(keys[j]), g_strdup(value));
+                g_hash_table_insert(date->holiday_solar, g_strdup(keys[j]),
+                                    g_strdup(value));
             }else if (g_ascii_strcasecmp(groups[i], "LUNAR") == 0 ) {
-                g_hash_table_insert(date->holiday_lunar, g_strdup(keys[j]), g_strdup(value));
+                g_hash_table_insert(date->holiday_lunar, g_strdup(keys[j]),
+                                    g_strdup(value));
             }else if (g_ascii_strcasecmp(groups[i], "WEEK") == 0 ) {
-                g_hash_table_insert(date->holiday_week, g_strdup(keys[j]), g_strdup(value));
+                g_hash_table_insert(date->holiday_week, g_strdup(keys[j]),
+                                    g_strdup(value));
             }
             j++;
         }
@@ -144,8 +170,7 @@ static void lunar_date_init_holiday (LunarDate *date)
     g_bytes_unref(bytes);
 }
 
-static void
-lunar_date_init (LunarDate *date)
+static void lunar_date_init (LunarDate *date)
 {
     lunar_date_init_i18n();
 
@@ -165,9 +190,10 @@ lunar_date_init (LunarDate *date)
 /**
  * lunar_date_new:
  *
- * Allocates a #LunarDate and initializes it. Free the return value with lunar_date_free().
+ * Allocates a `LunarDate`` and initializes it. Free the return value
+ * with [method@LunarDate.Date.free].
  *
- * Returns: a newly-allocated #LunarDate
+ * Returns: a newly-allocated `LunarDate`
  *
  * Since: 2.4.0
  **/
@@ -177,11 +203,10 @@ lunar_date_new (void)
     return g_object_new (LUNAR_TYPE_DATE, NULL);
 }
 
-static void
-lunar_date_set_property (GObject *object,
-                         guint    prop_id,
-                         const GValue *value,
-                         GParamSpec   *pspec)
+static void lunar_date_set_property (GObject *object,
+                                     guint    prop_id,
+                                     const GValue *value,
+                                     GParamSpec   *pspec)
 {
     switch (prop_id)
     {
@@ -226,14 +251,14 @@ GQuark lunar_date_error_quark (void)
 
 /**
  * lunar_date_set_solar_date:
- * @date: a #LunarDate.
+ * @date: a `LunarDate`
  * @year: year to set.
  * @month: month to set.
  * @day: day to set.
  * @hour: hour to set.
- * @error: location to store the error occuring, or NULL to ignore errors.
+ * @error: location to store the error occuring, or %NULL to ignore errors.
  *
- * Sets the solar year, month, day and the hour for a #LunarDate.
+ * Sets the solar year, month, day and the hour for a `LunarDate`
  *
  * Since: 2.4.0
  **/
@@ -246,28 +271,30 @@ void lunar_date_set_solar_date (LunarDate *date,
 {
     GError *calc_error = NULL;
 
-    if (year < BEGIN_YEAR || year > BEGIN_YEAR+NUM_OF_YEARS || (year == BEGIN_YEAR && month == 1))
+    if (year < BEGIN_YEAR || year > BEGIN_YEAR+NUM_OF_YEARS)
     {
-        g_set_error(error, LUNAR_DATE_ERROR, LUNAR_DATE_ERROR_YEAR, _("Year out of range.(from 1900 to 2100)."));
+        g_set_error(error, LUNAR_DATE_ERROR, LUNAR_DATE_ERROR_YEAR,
+                    _("Year out of range.(from 1900 to 2100)."));
         return;
     }
 
-    if (!g_date_valid_month(month))
-    {
-        g_set_error(error,
-                    LUNAR_DATE_ERROR,
-                    LUNAR_DATE_ERROR_MONTH,
-                    _("Month out of range."));
-        return;
+    if (year == BEGIN_YEAR && month == 1 && day < 31) {
+	g_set_error(error, LUNAR_DATE_ERROR, LUNAR_DATE_ERROR_MONTH,
+		    _("Day out of range.(Lunar start from 1900-01-31)."));
+	return;
+    }
+
+    if (!g_date_valid_month(month)) {
+	g_set_error(error, LUNAR_DATE_ERROR, LUNAR_DATE_ERROR_MONTH,
+		    _("The month value is invalid."));
+	return;
     }
 
     if (hour == 24) hour = 0;
     if (hour > 23)
     {
-        g_set_error(error,
-                    LUNAR_DATE_ERROR,
-                    LUNAR_DATE_ERROR_HOUR,
-                    _("Hour out of range."));
+        g_set_error(error, LUNAR_DATE_ERROR, LUNAR_DATE_ERROR_HOUR,
+		    _("Hour out of range."));
         return;
     }
 
@@ -285,26 +312,26 @@ void lunar_date_set_solar_date (LunarDate *date,
     }
     _cl_date_calc_ganzhi(date);
     _cl_date_calc_bazi(date);
-
 }
 
 /**
  * lunar_date_set_lunar_date:
- * @date: a #LunarDate.
+ * @date: a `LunarDate`
  * @year: year to set.
  * @month: month to set.
  * @day: day to set.
  * @hour: hour to set.
  * @isleap: indicate whether the month is a leap month.
- * @error: location to store the error occuring, or #NULL to ignore errors.
+ * @error: location to store the error occuring, or %NULL to ignore errors.
  *
- * Sets the lunar year, month, day and the hour for a #LunarDate. If the month is a leap month, you should set the isleap to TRUE.
+ * Sets the lunar year, month, day and the hour for a `LunarDate`. If the
+ * month is a leap month, you should set the isleap to TRUE.
  *
  * Since: 2.4.0
  **/
 void lunar_date_set_lunar_date (LunarDate *date,
                                 GDateYear year,
-		                        GDateMonth month,
+                                GDateMonth month,
                                 GDateDay day,
                                 guint8 hour,
                                 gboolean isleap,
@@ -368,7 +395,8 @@ void lunar_date_set_lunar_date (LunarDate *date,
  *
  * Since: 3.0.0
  **/
-void lunar_date_set_solar_holiday(LunarDate *date, GDateMonth month, GDateDay day, const gchar *holiday)
+void lunar_date_set_solar_holiday(LunarDate *date, GDateMonth month,
+                                  GDateDay day, const gchar *holiday)
 {
     gchar key[10];
 
@@ -393,7 +421,8 @@ void lunar_date_set_solar_holiday(LunarDate *date, GDateMonth month, GDateDay da
  *
  * Since: 3.0.0
  **/
-void lunar_date_set_lunar_holiday(LunarDate *date, GDateMonth month, GDateDay day, const gchar *holiday)
+void lunar_date_set_lunar_holiday(LunarDate *date, GDateMonth month,
+                                  GDateDay day, const gchar *holiday)
 {
     gchar key[10];
     g_return_if_fail( date != NULL);
@@ -418,7 +447,9 @@ void lunar_date_set_lunar_holiday(LunarDate *date, GDateMonth month, GDateDay da
  *
  * Since: 3.0.0
  **/
-void lunar_date_set_week_holiday(LunarDate *date, GDateMonth month, gint week_of_month, gint day_of_week, const gchar *holiday)
+void lunar_date_set_week_holiday(LunarDate *date, GDateMonth month,
+                                 gint week_of_month, gint day_of_week,
+                                 const gchar *holiday)
 {
     gchar key[10];
 
@@ -427,27 +458,78 @@ void lunar_date_set_week_holiday(LunarDate *date, GDateMonth month, gint week_of
     g_return_if_fail( day_of_week >= 0 && day_of_week <= 6);
     g_return_if_fail( holiday != NULL);
 
-    g_snprintf(key, sizeof(key), "%02d%01d%01d", date->solar->month, week_of_month, day_of_week);
+    g_snprintf(key, sizeof(key), "%02d%01d%01d", date->solar->month,
+               week_of_month, day_of_week);
     if (g_hash_table_lookup(date->holiday_week, key) != NULL) {
         g_hash_table_remove(date->holiday_week, key);
     }
     g_hash_table_insert(date->holiday_week, g_strdup(key), g_strdup(holiday));
 }
 
+/*
+ * return 1: off day
+ * return 0: work day
+ * return -1: no change.
+ */
+static gint check_statutory_holiday (GDateYear year,
+                                     GDateMonth month,
+                                     GDateDay day)
+{
+    GDate *gdate;
+    GDateWeekday weekday;
+    guint days;
+    guint16 num;
+    gboolean found = FALSE;
+    gint ret = -1;
+    gint length;
+    gint i;
+
+    if (year < 2000) {
+        return -1;
+    }
+
+    gdate = g_date_new_dmy (day, month, year);
+    days = g_date_get_day_of_year (gdate);
+
+    num = ((year - 2000) << 9) | days;
+    length = G_N_ELEMENTS (statutory_holidays);
+    for (i = 0; i < length; i++) {
+        if (statutory_holidays[i] == num) {
+            found = TRUE;
+            break;
+        }
+    }
+
+    if (found) {
+        weekday = g_date_get_weekday (gdate);
+        if (weekday == G_DATE_SATURDAY || weekday == G_DATE_SUNDAY) {
+            ret = 0;
+        } else {
+            ret = 1;
+        }
+    }
+
+    g_date_free (gdate);
+    return ret;
+}
 
 /**
  * lunar_date_get_real_holiday:
  * @date: a #LunarDate
  * @delimiter: used to join the holidays.
- * @full: Whether to display the full name of the holiday, the short name just used for calendar.
+ * @full: Whether to display the full name of the holiday, the short name
+ * just used for calendar.
  *
- * Returns the all holiday of the date, joined with the delimiter. The date must be valid.
+ * Returns the all holiday of the date, joined with the delimiter. The date
+ * must be valid.
  *
  * Returns:  a newly-allocated holiday string of the date or NULL.
  *
  * Since: 3.0.0
  **/
-static gchar* lunar_date_get_real_holiday (LunarDate *date, const gchar *delimiter, gboolean full)
+static gchar* lunar_date_get_real_holiday (LunarDate *date,
+                                           const gchar *delimiter,
+                                           gboolean full)
 {
     GString*      jieri;
     gint          i, weekday, weekth;
@@ -519,6 +601,24 @@ static gchar* lunar_date_get_real_holiday (LunarDate *date, const gchar *delimit
         }
     }
 
+    // statutory_holidays
+    i = check_statutory_holiday (date->solar->year, date->solar->month, date->solar->day);
+    if (i == 1) {
+        if (jieri->len > 0) jieri = g_string_append (jieri, delimiter);
+        if (full) {
+            jieri = g_string_append (jieri, "法定节假日");
+        } else {
+            jieri = g_string_append (jieri, "休息");
+        }
+    } else if (i == 0) {
+        if (jieri->len > 0) jieri = g_string_append (jieri, delimiter);
+        if (full) {
+            jieri = g_string_append(jieri, "法定工作日");
+        } else {
+            jieri = g_string_append(jieri, "上班");
+        }
+    }
+
     //jie2qi4
     if(atoi(yc) != date->solar->year)
     {
@@ -539,6 +639,7 @@ static gchar* lunar_date_get_real_holiday (LunarDate *date, const gchar *delimit
         }
         g_strfreev(jq_day);
     }
+
     if (jieri->len > 0 ) {
         return g_string_free(jieri, FALSE);
     }else{
@@ -616,12 +717,12 @@ gchar* lunar_date_get_calendar(LunarDate *date, gint max)
 
 /**
  * lunar_date_get_holiday:
- * @date: a #LunarDate
+ * @date: a `LunarDate``
  * @delimiter: used to join the holidays.
  *
  * Returns the all holiday of the date, joined with the delimiter. The date must be valid.
  *
- * Returns:  a newly-allocated holiday string of the date or NULL.
+ * Returns:  a newly-allocated holiday string of the date or %NULL.
  *
  * Since: 3.0.0
  **/
@@ -638,9 +739,9 @@ gchar* lunar_date_get_holiday (LunarDate *date, const gchar *delimiter)
  *
  * Returns the all holiday of the date, joined with the delimiter. The date must be valid.
  *
- * Returns: a newly-allocated holiday string of the date or NULL.
+ * Returns: a newly-allocated holiday string of the date or %NULL.
  *
- * Deprecated: 3.0.0: Use lunar_date_get_holiday() instead.
+ * Deprecated: 3.0.0: Use [method@LunarDate.Date.get_holiday] instead.
  **/
 gchar* lunar_date_get_jieri(LunarDate *date, const gchar *delimiter)
 {
@@ -650,13 +751,13 @@ gchar* lunar_date_get_jieri(LunarDate *date, const gchar *delimiter)
 
 /**
  * lunar_date_strftime:
- * @date: a #LunarDate
+ * @date: a `LunarDate`
  * @format: specify the output format.
  *
  * Use the given format to output a string, similar to strftime usage.
  * The available formats and outputs are as follows:
  *
- * |[<!-- language="C" -->
+ * ```
  * format="%(YEAR)年%(MONTH)月%(DAY)日%(HOUR)时", output="一九一○年二月十九日二时"   //大写公历
  * format="%(year)年%(month)月%(day)日%(hour)时", output="1910年2月19日2时"          //小写公历
  * format="%(NIAN)年%(YUE)月%(RI)日%(SHI)时",     output="庚戌年一月初十日丑时"      //大写农历(月份前带"闰"表示闰月)
@@ -665,9 +766,9 @@ gchar* lunar_date_get_jieri(LunarDate *date, const gchar *delimiter)
  * format="%(Y8)年%(M8)月%(D8)日%(H8)时",         output="庚戌年戊寅月乙卯日丁丑时"  //八字
  * format="%(shengxiao)",                        output="狗"                         //生肖
  * format="%(holiday)",                          output="雨水"                       //节日(节日、纪念日、节气等)
- * ]|
+ * ```
  * When using %(holiday), the output is automatically truncated to 3 utf8 characters or 4 ascii characters.
- * If you need all the holiday information, please use lunar_date_get_holiday () to get the output.
+ * If you need all the holiday information, please use [method@LunarDate.Date.get_holiday] to get the output.
  *
  * Returns: a newly-allocated output string, nul-terminated
  *
@@ -885,6 +986,16 @@ gchar* lunar_date_strftime (LunarDate *date, const char *format)
     return str;
 }
 
+/**
+ * lunar_date_get_constellation:
+ * @date: a `LunarDate``
+ *
+ * Returns the constellation name
+ *
+ * Returns: a newly-allocated string.
+ *
+ * Since: 3.0.0
+ **/
 gchar* lunar_date_get_constellation  (LunarDate *date)
 {
     GDate *gdate;
@@ -910,9 +1021,9 @@ gchar* lunar_date_get_constellation  (LunarDate *date)
 
 /**
  * lunar_date_free:
- * @date: a #LunarDate
+ * @date: a `LunarDate``
  *
- * Frees a #LunarDate returned from lunar_date_new().
+ * Frees a `LunarDate` returned from [ctor@LunarDate.Date.new].
  *
  * Since: 2.4.0
  **/
@@ -950,7 +1061,7 @@ static void _cl_date_calc_lunar(LunarDate *date, GError **error)
         return;
     }
     /* A lunar day begins at 11 p.m. */
-    if (date->solar->hour == 23)
+    if (date->solar->hour >= 23)
         date->days ++;
 
     _cl_date_days_to_lunar(date, &calc_error);
@@ -1041,13 +1152,15 @@ static void _date_calc_days_since_lunar_year (LunarDate *date, GError **error)
                     date->lunar->month, date->lunar->year);
         return;
     }
-    for (m=1; m<date->lunar->month; m++)
+    for (m=1; m<date->lunar->month; m++) {
         date->days+= date->lunar_month_days[m];
+    }
     if (leap_month
-            && ((date->lunar->month>leap_month)
-                || (date->lunar->isleap && (date->lunar->month==leap_month))
-               ))
+        && ((date->lunar->month>leap_month)
+        || (date->lunar->isleap && (date->lunar->month==leap_month))
+    )) {
         date->days += date->lunar_month_days[m++];
+    }
     date->days += date->lunar->day - 1;
 
     if (date->lunar->day > date->lunar_month_days[m])
@@ -1129,7 +1242,7 @@ static void _cl_date_days_to_solar(LunarDate *date, GError **error)
     /* offset is now the number of days from SolarFirstDate.year.1.1 */
 
     for (i=first_solar_date.year;
-            (i<first_solar_date.year+NUM_OF_YEARS) && (offset > 0);  i++)
+        (i<first_solar_date.year+NUM_OF_YEARS) && (offset > 0);  i++)
         offset -= 365 + leap(i);
     if (offset<0)
     {
@@ -1139,9 +1252,9 @@ static void _cl_date_days_to_solar(LunarDate *date, GError **error)
     if (i==(first_solar_date.year + NUM_OF_YEARS))
     {
         g_set_error(error, LUNAR_DATE_ERROR,
-                LUNAR_DATE_ERROR_DAY,
-                _("Year out of range. \"%d\""),
-                i);
+                    LUNAR_DATE_ERROR_DAY,
+                    _("Year out of range. \"%d\""),
+                    i);
         return;
     }
     date->solar->year = i;
@@ -1206,9 +1319,7 @@ static gint _cl_date_make_lunar_month_days(LunarDate *date, gint year)
             date->lunar_month_days[i] = days_in_lunar_month[code&0x1];
             code >>= 1;
         }
-    }
-    else
-    {
+    } else {
         /*
            There is a leap month (run4 yue4) L in this year.
            mday[1] contains the number of days in the 1-st month;
