@@ -1,22 +1,6 @@
-/* vi: set sw=4 ts=4 wrap ai expandtab: */
+/* SPDX-License-Identifier: LGPL-2.1-only */
 /*
- * date-daemon.c: This file is part of lunar-date.
- *
- * Copyright (C) 2016-2021 yetist <yetist@gmail.com>
- *
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this library; if not, see <http://www.gnu.org/licenses/>.
+ * Copyright (c) 2007-2025 yetist <yetist@gmail.com>
  * */
 
 
@@ -92,7 +76,7 @@ gboolean date_get_holiday (BusLunarDate *object,
     daemon = DATE_DAEMON (user_data);
     lunar_date_set_solar_date(daemon->date, year, month, day, (guint8) hour, &error);
     if (error != NULL ) {
-        g_dbus_method_invocation_return_error (invocation, g_quark_from_static_string(DATE_DBUS_NAME), 1, "%s", error->message);
+        g_dbus_method_invocation_return_error (invocation, g_quark_from_static_string(DATE_DBUS_NAME), error->code, "%s", error->message);
         return FALSE;
     }
     holiday = lunar_date_get_holiday(daemon->date, delimiter);
@@ -119,9 +103,9 @@ gboolean date_get_strftime (BusLunarDate *object,
     GError *error = NULL;
 
     daemon = DATE_DAEMON (user_data);
-    lunar_date_set_solar_date(daemon->date, year, month, day , hour, &error);
+    lunar_date_set_solar_date(daemon->date, year, month, day, (guint8) hour, &error);
     if (error != NULL ) {
-        g_dbus_method_invocation_return_error (invocation, g_quark_from_static_string(DATE_DBUS_NAME), 1, "%s", error->message);
+        g_dbus_method_invocation_return_error (invocation, g_quark_from_static_string(DATE_DBUS_NAME), error->code, "%s", error->message);
         return FALSE;
     }
     result = lunar_date_strftime(daemon->date, format);
@@ -144,9 +128,9 @@ gboolean date_get_calendar(BusLunarDate *object,
     GError *error = NULL;
 
     daemon = DATE_DAEMON (user_data);
-    lunar_date_set_solar_date(daemon->date, year, month, day , hour, &error);
+    lunar_date_set_solar_date(daemon->date, year, month, day , (guint8) hour, &error);
     if (error != NULL ) {
-        g_dbus_method_invocation_return_error (invocation, g_quark_from_static_string(DATE_DBUS_NAME), 1, "%s", error->message);
+        g_dbus_method_invocation_return_error (invocation, g_quark_from_static_string(DATE_DBUS_NAME), error->code, "%s", error->message);
         return FALSE;
     }
     result = lunar_date_get_calendar(daemon->date, max);
@@ -171,7 +155,7 @@ static void bus_acquired_handler_cb (GDBusConnection *connection,
     g_signal_connect (daemon->skeleton, "handle-calendar", G_CALLBACK (date_get_calendar), daemon);
 
     exported = g_dbus_interface_skeleton_export (G_DBUS_INTERFACE_SKELETON (daemon->skeleton),
-            connection, DATE_DBUS_PATH, &error);
+                                                 connection, DATE_DBUS_PATH, &error);
 
     if (!exported)
     {
@@ -277,12 +261,12 @@ static void date_daemon_class_init (DateDaemonClass *class)
 
     properties[PROP_LOOP] =
         g_param_spec_pointer("loop", "loop", "loop",
-                G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE |
-                G_PARAM_STATIC_STRINGS);
+                             G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE |
+                             G_PARAM_STATIC_STRINGS);
     properties[PROP_REPLACE] =
         g_param_spec_boolean ("replace", "replace", "replace", FALSE,
-                G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE |
-                G_PARAM_STATIC_STRINGS);
+                              G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE |
+                              G_PARAM_STATIC_STRINGS);
 
     g_object_class_install_properties (gobject_class, LAST_PROP, properties);
 }
