@@ -1,23 +1,7 @@
-/* vi: set sw=4 ts=4 wrap ai expandtab: */
+/* SPDX-License-Identifier: LGPL-2.1-only */
 /*
- * lunar-date.c: This file is part of lunar-date.
- *
- * Copyright (C) 2015-2021 yetist <yetist@gmail.com>
- *
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this library; if not, see <http://www.gnu.org/licenses/>.
- * */
+ * Copyright (C) 2007-2025 yetist <yetist@gmail.com>
+ */
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
@@ -39,8 +23,7 @@ void show_year_month_cal(BusLunarDate *date, gint year, gint month);
 void show_year_cal(BusLunarDate *date, gint year);
 void show_day_hour_info(BusLunarDate *date, gint year, gint month, gint day, gint hour);
 
-static GOptionEntry entries[] =
-{
+static GOptionEntry entries[] = {
     { "format", 'f', 0, G_OPTION_ARG_STRING, &format, N_("Format for date output"), "N" },
     { NULL }
 };
@@ -49,7 +32,7 @@ void show_year_month_cal(BusLunarDate *date, gint year, gint month)
 {
     guint i;
     guint8 days;
-    gchar *string;
+    gchar *string = NULL;
     GError *error = NULL;
     GDate *gd;
     gchar* lunar[32] = {0};
@@ -178,9 +161,16 @@ void show_day_hour_info(BusLunarDate *date, gint year, gint month, gint day, gin
                                        NULL,
                                        &error);
     if (error != NULL) {
-        g_print("%d\n", error->code);
         if (error->code == LUNAR_DATE_ERROR_YEAR) {
             g_printerr (_("Unable to get cal: %s\n"), _("Year out of range.(from 1900 to 2100)."));
+        } else if (error->code == LUNAR_DATE_ERROR_MONTH) {
+            g_printerr (_("Unable to get cal: %s\n"), _("Month out of range.(Start from 1900-02)."));
+        } else if (error->code == LUNAR_DATE_ERROR_DAY) {
+            g_printerr (_("Unable to get cal: %s\n"), _("Day out of range."));
+        } else if (error->code == LUNAR_DATE_ERROR_HOUR) {
+            g_printerr (_("Unable to get cal: %s\n"), _("Hour out of range."));
+        } else if (error->code == LUNAR_DATE_ERROR_LEAP) {
+            g_printerr (_("Unable to get cal: %s\n"), _("Leap month in year error.\n"));
         }
         g_error_free(error);
         return;
@@ -261,9 +251,12 @@ int main (int argc, char **argv)
         show_year_cal(date, year);
     else if (i == 3)
         show_year_month_cal(date, year, month);
-    else if (i == 4)
-        show_day_hour_info(date, year, month, day, -1);
-    else
+    else {
+        if (year == 1900 && month == 1 && day < 31) {
+            g_print(_("Error: %s\n"), _("Day out of range.(Lunar start from 1900-01-31)."));
+            return EXIT_SUCCESS;
+        }
         show_day_hour_info(date, year, month, day, hour);
+    }
     return EXIT_SUCCESS;
 }
